import * as server_main from '@minecraft/server'
import { ActionFormData, MessageFormData, ModalFormData} from '@minecraft/server-ui';
import * as data_convert from './data_convert.js';
import * as entity_functions from './entity_functions.js';
import * as command_controller from './command_controller.js';
import * as commander_controller from './commander_controller.js';


const title_display = {
    fadeInDuration: 0,
    stayDuration: 0,
    fadeOutDuration: 0
};

//UI SPECIFIC
export function UI_SELECTSIDES(player) {
    player.runCommandAsync(`tag @s remove select_faction`)
    dialogueCommand(player, 'select_sides')
}
export function UI_COMMANDPOST(player, command_entity) {
    let command_commander = entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(command_entity, 'player_host'));
    let command_commanderID = entity_functions.get_entity_objective_score(command_entity, 'player_host');
    let command_factionID = entity_functions.get_entity_objective_score(command_entity, 'chosen_faction');
    let command_popcount = command_controller.get_command_pop(command_commander);
    let command_recruitcount = entity_functions.get_entity_objective_score(command_commander, 'command_count');
    let command_limit = command_controller.get_command_limit(command_commander);

    let form = new ActionFormData()
        .title("Faction Command Post")
        .body("Commander: " + data_convert.PlayerIDtoName(command_commanderID).toString() + "\n" + 
            "Faction: " + data_convert.FactionIDtoName(command_factionID).toString() + "\n" + 
            "Command Post Size: " + data_convert.CommandSizeID(command_limit).toString() + "\n" + 
            "Population: " + ((command_popcount == command_limit) ? "§2" : "") + command_popcount.toString() + "/" + command_limit.toString() + "§f\n" + 
            "Recruited Entities: " + command_recruitcount.toString() + "\n" + 
            "Command ID: " + command_entity.id.toString() + "\n")
        .button("Close")
        ;
    form.show(player).then(r => {
        // The code after player respond/close the form
    }).catch((e) => {
        console.error(e, e.stack);
    });

}
export function UI_UpdateCommandHud(player) {
    var playerCommandLevel = entity_functions.get_entity_objective_score(player, "command_level").toString();
    var playerCommandPoints = entity_functions.get_entity_objective_score(player, "command_points").toString();
    var RankName = commander_controller.getLVLName(playerCommandLevel, entity_functions.get_entity_objective_score(player, 'chosen_faction'));

    if (entity_functions.get_entity_objective_score(player, 'command_count') == -1){
        player.runCommandAsync('scoreboard players set @s command_count 0');
    }
    if (entity_functions.get_entity_objective_score(player, 'command_population') == -1){
        player.runCommandAsync('scoreboard players set @s command_population 0');
    }
    if (player){
        player.onScreenDisplay.setTitle('command_updateXP' + RankName + ' (' + playerCommandLevel + ')   ' + playerCommandPoints + '/' + commander_controller.getNextLVLXP(playerCommandLevel).toString(), title_display) ;//Up to 12
        player.removeTag("command_settitlebar")   ;
    }
}
//LOCAL COMMANDS
function dialogueCommand(player, dialogue_name) {
    player.runCommandAsync('event entity @s minecraft:add_npc')
    player.runCommandAsync(`dialogue open @s @s ${dialogue_name}`)
    player.runCommandAsync('event entity @s minecraft:add_npc')
}
function infoMessage(messageTXT, titleTXT, buttonTXT) {
    return new ActionFormData()
        .title(titleTXT)
        .body(messageTXT)
        .button(buttonTXT)
}
