import * as server_main from '@minecraft/server'
import * as uicontroller from './controllers/ui_controller.js';
import * as recruitment_controller from './controllers/recruitment_controller.js';
import * as command_controller from './controllers/command_controller.js';
import * as entity_functions from './controllers/entity_functions.js';
import * as data_convert from './controllers/data_convert.js';
import * as commander_controller from './controllers/commander_controller.js';
server_main.system.runInterval(() => {
    for (const player of server_main.world.getAllPlayers()) {
        if (player.hasTag("select_faction")) {
            uicontroller.UI_SELECTSIDES(player)  ;          
        }
        else if (player.hasTag("command_ui")) {
            command_controller.open_commandpost(player, entity_functions.get_nearest_entityTAG_OBJECTIVE(player, 25, "command_post", "player_host", parseInt(player.id.toString().slice(5))))     ;       
        }
        else if (player.hasTag("update_post")) {
            command_controller.update_commandpost(player);
        }
        else if (player.hasTag("command_settitlebar")) {
            uicontroller.UI_UpdateCommandHud(player);
        }
        else if (player.hasTag("initing_mordor")) {
            player.runCommandAsync("title @s actionbar Generating Mordor...") ;
        }
    }
});

server_main.world.afterEvents.worldInitialize.subscribe((event) => {   
    //COMMANDER/ENTITY CHOSEN FACTION
    if (!server_main.world.scoreboard.getObjective('chosen_faction'))
    {
        server_main.world.scoreboard.addObjective('chosen_faction', 'Faction');
    }
    //CONTAINS THE PLAYERS COMMANDER LEVEL
    if (!server_main.world.scoreboard.getObjective('command_level'))
    {
        server_main.world.scoreboard.addObjective('command_level', 'Commander Level');
    }
    //CONTAINS THE COMMAND "XP" OF THE PLAYER OR THE POPULATION REQUIREMENT OF A RECRUITED ENTITY
    if (!server_main.world.scoreboard.getObjective('command_points'))
    {
        server_main.world.scoreboard.addObjective('command_points', 'Commander Points');
    }
    //CONTAINS THE POPULATION HANDLER OF THE PLAYER
    if (!server_main.world.scoreboard.getObjective('command_population'))
    {
        server_main.world.scoreboard.addObjective('command_population', 'Current Population');
    }
    //CONTAINS THE PLAYERS AMOUNT OF RECRUITED ENTITIES
    if (!server_main.world.scoreboard.getObjective('command_count'))
    {
        server_main.world.scoreboard.addObjective('command_count', 'Current Recruited');
    }
    //COMMAND POSTS ID FOR PLAYER
    if (!server_main.world.scoreboard.getObjective('command_post_id'))
    {
        server_main.world.scoreboard.addObjective('command_post_id', 'Command Post');
    }
    //FOR THE ENTITIES THAT HAVE A PLAYER HOST OR COMMAND POST
    if (!server_main.world.scoreboard.getObjective('player_host'))
    {
        server_main.world.scoreboard.addObjective('player_host', 'Host');
    }
})

server_main.world.afterEvents.playerSpawn.subscribe(event => {
    event.player.addTag("command_settitlebar");
})



server_main.world.afterEvents.blockPlace.subscribe((event) => {
    if (event.block.typeId == "lotr:mordor_commandpost"){
        let loc = event.block.x.toString() + ' ' + event.block.y.toString() + ' ' + event.block.z.toString()
        if (data_convert.faction_name(entity_functions.get_entity_objective_score(event.player, "chosen_faction")) != 'mordor'){
            event.player.sendMessage("Place a command post of your faction...")
            event.player.runCommandAsync('setblock ' + loc + ' air')
            event.player.runCommandAsync('give @p lotr:command_post_mordor_item 1')   
        }
        else if (entity_functions.check_valid_score(event.player, "command_post_id")){
            event.player.sendMessage("Delete your old command post before trying to place a new one...")
            event.player.runCommandAsync('setblock ' + loc + ' air')
            event.player.runCommandAsync('give @p lotr:command_post_mordor_item 1')           
        }
        else{
            command_controller.init_commandpost(event.player, event.block, 'mordor')   
        }    
    }
    if (event.block.typeId == "lotr:gondor_commandpost"){
        let loc = event.block.x.toString() + ' ' + event.block.y.toString() + ' ' + event.block.z.toString()
        if (data_convert.faction_name(entity_functions.get_entity_objective_score(event.player, "chosen_faction")) != 'gondor'){
            event.player.sendMessage("Place a command post of your faction...")
            event.player.runCommandAsync('setblock ' + loc + ' air')
            event.player.runCommandAsync('give @p lotr:command_post_gondor_item 1')   
        }
        else if (entity_functions.check_valid_score(event.player, "command_post_id")){
            event.player.sendMessage("Delete your old command post before trying to place a new one!")
            event.player.runCommandAsync('setblock ' + loc + ' air')
            event.player.runCommandAsync('give @p lotr:command_post_gondor_item 1')           
        }
        else{
            command_controller.init_commandpost(event.player, event.block, 'gondor')   
        }  
    }

    if (event.block.typeId == "lotr:wattle_daub"){
        const loc = [event.block.x, event.block.y, event.block.z];
        feature_commands.splatter_ground("lotr:wattle_daub", 16, loc, event.player)
    }
})
server_main.world.afterEvents.blockBreak.subscribe((event) => {
    if (event.brokenBlockPermutation.type.id.toString().includes("_commandpost")){
        let loc = event.block.location
        var command_entity = entity_functions.get_nearest_entityTYPE_LOC(loc, 2, "lotr:command_post")
        var command_entity_host = entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(command_entity, "player_host"))

        if (!command_entity){
            event.player.sendMessage('Invalid command entity')
        }
        else if (!command_entity_host){
            event.player.sendMessage('Invalid command entity host')
        }
        else
        {
            if (command_entity_host.id == event.player.id){
                event.player.runCommandAsync('scoreboard players set @s command_post_id 0')
                event.player.sendMessage('Your command post is now erased...')
            }
            else if (command_entity_host.id != event.player.id)
            {
                command_entity_host.runCommandAsync('scoreboard players set @s command_post_id 0')
                event.player.sendMessage('Erased ' + command_entity.name.toString() + '...')
                command_entity_host.sendMessage('Your command post is now erased...')
            }
            else{
                event.player.sendMessage('Something Went Wrong...')
            }
        }

        
    }

    //OLD
   /* if (event.brokenBlockPermutation.type.id == "lotr:gondor_commandpost"){
        let loc = new server_main.Location(event.block.x, event.block.y, event.block.z)
        var entity = entity_functions.get_nearest_entityTYPE_OBJECTIVE_LOC(loc, 2, "lotr:command_post", "player_host", event.player.id.toString().slice(5)) //not getting entity

        if (entity.id.toString().slice(5) == entity_functions.get_entity_objective_score(event.player, "command_post_id").toString()){ 
            event.player.runCommandAsync('scoreboard players set @s command_post_id 0')
            event.player.sendMessage('Your command post is now erased...')
        }
    }*/
   
})





server_main.world.afterEvents.entitySpawn.subscribe((event) =>{
    if (!event.entity){return}
    //SPAWNING A TAMED ENTITY FROM RECRUITING EGG
    if (entity_functions.isEntityRecruited(event.entity) && event.entity.hasComponent("minecraft:is_tamed")){
        //IGNORE IF STATIC
        if (event.entity.typeId == "throwable_pretamed_entity" || event.entity.typeId == "throwable_pretamed_entity_static"){return}
        //IGNORE IF ALREADY ASSIGNED A PLAYER HOST     
        if (entity_functions.check_valid_score(event.entity, "player_host")){return}
        //Find nearest player with same faction to be commander
        let commander = entity_functions.get_nearest_entityTYPE_OBJECTIVE(event.entity, 8, "minecraft:player", "chosen_faction", entity_functions.get_entity_objective_score(event.entity, "chosen_faction"))
        if (!commander){ 
            server_main.world.sendMessage("No valid Commander found...")
            recruitment_controller.fail_spawn(event.entity) 
            return 
        }
        //Check if population limit has been reached
        var populationCost = entity_functions.get_entity_objective_score(event.entity, "command_population")
        if (recruitment_controller.recruit_check(commander, event.entity, populationCost)){
            event.entity.runCommandAsync('scoreboard players set @s player_host ' + commander.id.toString().slice(5))
            commander.runCommandAsync('scoreboard players add @s command_population ' + populationCost)
            commander.runCommandAsync('scoreboard players add @s command_count 1')
        }
        else
        {
            recruitment_controller.fail_spawn_c(event.entity, commander)
            return 
        }
    }
})
server_main.world.afterEvents.entityHurt.subscribe((event) =>{

    /*if (event.hurtEntity.hasTag("spearWeakness") && event.damageSource.damagingEntity.getComponent("minecraft:inventory")){
        event.damageSource.damagingEntity.sendMessage("Item Equipped: " + event.damageSource.damagingEntity.getComponent("minecraft:inventory").container.getItem(98).typeId)
        event.hurtEntity.applyDamage(Number(event.damage), event.damageSource.damagingEntity)
    }*/
                
    //On a death
    if (!event.hurtEntity.getComponent("health")){return}
    let health = event.hurtEntity.getComponent("health").currentValue
    if (health <= 0){

        //Cant lock in an attacker or dying entity
        if (!event.damageSource.damagingEntity || !event.hurtEntity){return}

        //CALLED IF KILLED ENTITY HAS A SCORE FOR XP TO GAIN (HAS COMMAND POINTS SCORE + CHECK FACTION) - Faction 88 = WILD
        //Give XP to proper killer IF warented by faction (ADD IF TAMED ENTITY KILLS, GET TAG[1] PID AND ADD XP TO THAT PLAYER)
        if (entity_functions.check_valid_score(event.hurtEntity, "command_points")){
            var gained_xp = 0

            //If killing a player, get a portion of their XP
            if (event.hurtEntity.typeId == "minecraft:player"){
                gained_xp = entity_functions.get_entity_objective_score(event.hurtEntity, "command_level")
                gained_xp *= 8.2
            }
            else{
                gained_xp = entity_functions.get_entity_objective_score(event.hurtEntity, "command_points")
            }
            var gain_side = data_convert.FactionSIDEINVERT(entity_functions.get_entity_objective_score(event.hurtEntity, "chosen_faction"))
            var damager_side = data_convert.FactionSIDE(entity_functions.get_entity_objective_score(event.damageSource.damagingEntity, "chosen_faction"))
            
            if (damager_side == gain_side || entity_functions.get_entity_objective_score(event.hurtEntity, "chosen_faction") == 88){
                //server_main.world.say("giving XP")
                //DIRECT TO PLAYER
                if (event.damageSource.damagingEntity.typeId == "minecraft:player"){//give player xp for killing entity
                    commander_controller.modify_commanderXP(event.damageSource.damagingEntity, gained_xp)
                }
                //A TAMED ENTITY KILLS IT
                else if (entity_functions.check_valid_score(event.damageSource.damagingEntity, "player_host")){
                    gained_xp *= 0.7
                    var playerHost = entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(event.damageSource.damagingEntity, "player_host"))            
                    commander_controller.modify_commanderXP(playerHost, Math.round(Number(gained_xp)))
                }
            }
        }

        //Update the population of the Commander of the tamed entity IF entity is tamed player host
        if (entity_functions.check_valid_score(event.hurtEntity, "player_host")){
            var gained_pop = entity_functions.get_entity_objective_score(event.hurtEntity, "command_population")
            event.damageSource.damagingEntity.runCommandAsync(('scoreboard players remove ' + entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(event.hurtEntity, "player_host")).nameTag + ' command_population ' + gained_pop))
            event.damageSource.damagingEntity.runCommandAsync(('scoreboard players remove ' + entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(event.hurtEntity, "player_host")).nameTag + ' command_count 1'))          
        }



        //If a command post is destroyed, update the players command_post_id score
        if (event.hurtEntity.typeId == "lotr:command_post"){
            //player.runCommandAsync('scoreboard players set @e[type=lotr:command_post, c=1] player_host ' + (player.id.toString().slice(5)))
            var player = entity_functions.get_playerBy_ID(entity_functions.get_entity_objective_score(event.hurtEntity, "player_host"))
            world.sendMessage(player.nameTag.toString())
            player.runCommandAsync('scoreboard players set @s command_post_id 0')
        }
    }  
})

server_main.world.afterEvents.dataDrivenEntityTriggerEvent.subscribe((event) =>{
    if (event.id.startsWith("bind") || (event.id.startsWith("tameable") && event.entity.hasTag("TAMEABLELVLREQCHECK"))){
        let lvlReq = 0;
        for (const tag of event.entity.getTags()) {
            if (tag.startsWith("lvlReq")){
                lvlReq = Number(tag.match(/\d+/))
            }
        }
        let player = entity_functions.get_nearest_entityTYPE(event.entity, 10, "minecraft:player")
        if (data_convert.FactionSIDE(entity_functions.get_entity_objective_score(player, "chosen_faction")) == "DARK"){
            //player.runCommandAsync('sendMessage @s Taming Cost: ' + lvlReq.toString() + ' Player: ' + player.name)
            if (entity_functions.get_entity_objective_score(player, "command_level") >= lvlReq)
            {
                player.runCommandAsync('sendMessage @s Binding success...')
                //Continue tame
            }
            else{
                player.runCommandAsync('sendMessage @s Your commander level is not high enough to tame this creature...')
                //fail tame
                event.entity.triggerEvent("auto_fail_tame")
            }
        }
        else
        {
            player.runCommandAsync('sendMessage @s Your faction is unable to tame this creature...')
            //fail tame
            event.entity.triggerEvent("auto_fail_tame")
        }
    }
})